==> dancingparticles0.2.tgz: dancingparticles0.2/gl.cc <==

#include <gtk/gtk.h>

#include <GL/gl.h>
#include <GL/glx.h>
#include <GL/glu.h>

#include <pthread.h>
#include <sched.h>


#include <math.h>
#include <unistd.h>
#include <stdio.h>
#include <iostream.h>
#include "xmms/xmmsctrl.h"
#include "xmms/configfile.h"
#include "xmms/plugin.h"
#include "gl.h"
#include "gl_plugin.h"
#include "etoile.h"



extern VisPlugin bezier_vplugin;
extern GLfloat heights[64];
extern pthread_mutex_t height_mutex;
extern gboolean paused;

typedef struct _basis {
  GLfloat poly[4];
  GLfloat deriv[4];
} basis;

GLfloat surface[4][4][4][4];

#define camera_path_len 4

GLfloat camera_path[camera_path_len][4][4] =
{{{0, 0, 5.0, 0}, {2.5, 0, 5.0, 0}, {5.0, 0, 2.5, 0}, {5.0, 0, 0, 0}},
{{5.0, 0, 0, 0}, {5.0, 0, -2.5, 0}, {2.5, 0, -5.0, 0}, {0, 0, -5.0, 0}},
{{0, 0, -5.0, 0}, {-2.5, 0, -5.0, 0}, {-5.0, 0, -2.5, 0}, {-5.0, 0, 0, 0}},
{{-5.0, 0, 0, 0}, {-5.0, 0, 2.5, 0}, {-2.5, 0, 5.0, 0}, {0, 0, 5.0, 0}}};

static GLfloat camera_pos[4];
static int camera_index;
static GLfloat camera_u;
GLuint texture[1];

int sortedtab[ptsNumMax];


void LoadTexture()
{
  static GLubyte imageData[64][64][3];
  for(int i=0;i<64;i++){
    for(int j=0;j<64;j++){
      float x= 255*sqrt((i-32)*(i-32)+(j-32)*(j-32))/32;
      //x= 200*(sin(x/56)/(x/56)+0.23);
      //x= 35000*(1/(x+100)-0.0025);
      x=15000*(1/(x+50)-0.003);
      if(x<0)
	x=0;
      imageData[i][j][0]= (GLubyte)x;
      imageData[i][j][1]= (GLubyte)x;
      imageData[i][j][2]= (GLubyte)x;
      
    }
  }
 // Create Texture	
  glGenTextures(1, &texture[0]);
  glBindTexture(GL_TEXTURE_2D, texture[0]);   // 2d texture (x and y size)

  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture
  
  // 2d texture, level of detail 0 (normal), 3 components (red, green, blue), x size from image, y size from image, 
  // border 0 (normal), rgb color data, unsigned byte data, and finally the data itself.
  glTexImage2D(GL_TEXTURE_2D, 0, 3, 64, 64, 0, GL_RGB, GL_UNSIGNED_BYTE,&imageData[0][0][0] );
 
}
void init_gl(void)
{
  glViewport(0, 0, 640, 480);
  LoadTexture();
  glEnable(GL_TEXTURE_2D);			// Enable Texture Mapping
  glClearColor(0.0f, 0.0f, 1.0f, 0.0f);	// Clear The Background Color To Blue 
  glClearDepth(1.0);				// Enables Clearing Of The Depth Buffer
  glDepthFunc(GL_LESS);			// The Type Of Depth Test To Do
  glEnable(GL_DEPTH_TEST);			// Enables Depth Testing
  glShadeModel(GL_SMOOTH);			// Enables Smooth Color Shading
    
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();				// Reset The Projection Matrix
    
    
  gluPerspective(45.0f,(GLfloat)640/(GLfloat)480,0.1f,100.0f);	// Calculate The Aspect Ratio Of The Window
    
  glMatrixMode(GL_MODELVIEW);

  for(int i=0;i<ptsNumMax;i++)
    sortedtab[i]=i;
}

static void calc_basis(basis *b, GLfloat u)
{
  GLfloat v, v2, u2, ut3;
  v = 1.0 - u;;
  v2 = v * v;
  u2 = u * u;
  b->poly[0] = v * v2;
  b->poly[1] = 3.0 * u * v2;
  b->poly[2] = 3.0 * u2 * v;
  b->poly[3] = u * u2;
  ut3 = 3.0 * u;
  b->deriv[0] = -3.0 * v2;
  b->deriv[1] = (ut3 - 1.0) * (ut3 - 3.0);
  b->deriv[2] = ut3 * (2.0 - ut3);
  b->deriv[3] = 3.0 * u2;
}

static void calc_normal(GLfloat p[4])
{
  GLfloat mag;
  mag = sqrtf(p[0]*p[0] + p[1]*p[1] + p[2]*p[2]);
  p[0] = p[0] / mag;
  p[1] = p[1] / mag;
  p[2] = p[2] / mag;
}

static void eval_cubic_curve_vertex(GLfloat curve[4][4], basis *bu, GLfloat p[4])
{
  int i;
  p[0] = p[1] = p[2] = 0;
  for (i = 0; i < 4; i++) {
    p[0] += curve[i][0] * bu->poly[i];
    p[1] += curve[i][1] * bu->poly[i];
    p[2] += curve[i][2] * bu->poly[i];
  }
}

static void eval_cubic_curve_normal(GLfloat curve[4][4], basis *bu, GLfloat p[4])
{
  int i;
  p[0] = p[1] = p[2] = 0;
  for (i = 0; i < 4; i++) {
    p[0] += curve[i][0] * bu->deriv[i];
    p[1] += curve[i][1] * bu->deriv[i];
    p[2] += curve[i][2] * bu->deriv[i];
  }
  calc_normal(p);
}

static void eval_cubic_surface_vertex(GLfloat patch[4][4][4], basis *bu, basis *bv,
				      GLfloat p[4])
{
  int i, j;
  p[0] = p[1] = p[2] = 0;
  for (i = 0; i < 4; i++) {
    for (j = 0; j < 4; j++) {
      p[0] += patch[i][j][0] * (bu->poly[i] * bv->poly[j]);
      p[1] += patch[i][j][1] * (bu->poly[i] * bv->poly[j]);
      p[2] += patch[i][j][2] * (bu->poly[i] * bv->poly[j]);
    }
  }
}

static void eval_cubic_surface_normal(GLfloat patch[4][4][4], basis *bu, basis *bv,
				      GLfloat p[4])
{
  int i, j;
  p[0] = p[1] = p[2] = 0;
  for (i = 0; i < 4; i++) {
    for (j = 0; j < 4; j++) {
      p[0] += patch[i][j][0] * (bu->deriv[i] * bv->deriv[j]);
      p[1] += patch[i][j][1] * (bu->deriv[i] * bv->deriv[j]);
      p[2] += patch[i][j][2] * (bu->deriv[i] * bv->deriv[j]);
    }
  }
  calc_normal(p);
}

static void eval_cubic_surface_color(GLfloat patch[4][4][4], basis *bu, basis *bv,
				     GLfloat p[4])
{
  int i, j;
  p[0] = p[1] = p[2] = 0;
  for (i = 0; i < 4; i++) {
    for (j = 0; j < 4; j++) {
      p[0] += patch[i][j][0] * (bu->poly[i] * bv->poly[j]);
      p[1] += patch[i][j][1] * (bu->poly[i] * bv->poly[j]);
      p[2] += patch[i][j][2] * (bu->poly[i] * bv->poly[j]);
    }
  }
}

static void draw_patch(GLfloat patch[4][4][4][4], int u_steps, int v_steps)
{
  int u_freq, v_freq;
  basis bu, bv;
  GLfloat surface[4][u_steps+1][v_steps+1][4];
  int i, j;

  u_freq = u_steps + 1;
  v_freq = v_steps + 1;

  for (i = 0; i < u_freq; i++) {
    for (j = 0; j < v_freq; j++) {
      calc_basis(&bu, i / (float) u_steps);
      calc_basis(&bv, j / (float) v_steps);
      eval_cubic_surface_vertex(patch[0], &bu, &bv, surface[0][i][j]);
      eval_cubic_surface_normal(patch[1], &bu, &bv, surface[1][i][j]);
      eval_cubic_surface_color (patch[2], &bu, &bv, surface[2][i][j]);
    }
  }

  glEnable(GL_AUTO_NORMAL);
  for (i = 0; i < (v_freq-1); i++) {
    glBegin(GL_QUAD_STRIP);
    for (j = 0; j < u_freq; j++) {
      glColor3fv(surface[2][j][i]);
      glVertex3fv(surface[0][j][i]);
      glNormal3fv(surface[1][j][i]);
      glColor3fv(surface[2][j][i+1]);
      glVertex3fv(surface[0][j][i+1]);
      glNormal3fv(surface[1][j][i+1]);
    }
    glEnd();
  }
}

void draw_gl(void)
{

  basis bu;

  glClearColor(0,0,0,0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(-1, 1, -1, 1, 1.5, 40);

  calc_basis(&bu, camera_u);
  eval_cubic_curve_vertex(camera_path[camera_index], &bu, camera_pos);
  gluLookAt(camera_pos[0], bezier_cfg.camera_height, camera_pos[2],
	    0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();


  etoileLoop();

  if (!paused) {
    camera_u += bezier_cfg.camera_velocity / 500;
    if (camera_u > 1.0) {
      camera_index = (camera_index + 1) % camera_path_len;
      camera_u -= 1.0;
    }
  }
}

void init_surface(void)
{
  int x, z;
  GLfloat r_base, g_base;

  /* initialize vertices */
  for (z = 0; z < 4; z++) {
    for (x = 0; x < 4; x++) {
      surface[0][z][x][0] = x-((4-1)/2)-0.5; /* x */
      surface[0][z][x][1] = 0;               /* y */
      surface[0][z][x][2] = z-((4-1)/2)-0.5; /* z */
      surface[0][z][x][3] = 1;               /* w */
    }
  }

  /* initialize colors */
  for(z = 0; z < 4; z++) {
    r_base = z * (1.0 / 3.0);
    g_base = 1.0 - r_base;
    for(x = 0; x < 4; x++) {
      surface[2][3-z][x][0] = r_base;                        /* r */
      surface[2][3-z][x][1] = g_base - (x * (g_base / 3.0)); /* g */
      surface[2][3-z][x][2] = x * (1.0 / 3.0);               /* b */
      surface[2][3-z][x][3] = 0.0;                           /* a */
    }
  }

  /* initialize camera */
  camera_u = 0.0;
  camera_index = 0;

  /*
  glPixelStorei(GL_UNPACK_ALIGNMENT,1);
  glTexImage2D(GL_TEXTURE_2D,0,2,64,64,0,GL_RGB,GL_UNSIGNED_BYTE,&imageData[0][0][0]);
  glTexParameterf(GL_TEXTURE_2D,
		  GL_TEXTURE_WRAP_S,GL_CLAMP);
  glTexParameterf(GL_TEXTURE_2D,
		  GL_TEXTURE_WRAP_T,GL_CLAMP);
  glTexParameterf(GL_TEXTURE_2D,
		  GL_TEXTURE_MAG_FILTER,GL_NEAREST);
  glTexParameterf(GL_TEXTURE_2D,
		  GL_TEXTURE_MIN_FILTER,GL_NEAREST);
  glTexEnvf(GL_TEXTURE_ENV,
	    GL_TEXTURE_ENV_MODE,GL_DECAL);
  glEnable(GL_TEXTURE_2D);
  glShadeModel(GL_FLAT);
  */
}



void DrawProg(int i,int n)
{
}

extern int newline[ptsNumMax];
extern int numCenters;

void DrawPt(int ii)
{
 
  int i = sortedtab[ii];
  GLfloat colors[][3]={ 
    {1.,0.2,0.2   }
  ,{0.2,1.,0.2}
  ,{0.4,0.4,1}};
  static int r;
  int i1 = (i+1)%ptsNum;
  int linked = 0;//(((p.strombo==0 || r%p.strombo ==1 )&& (p.mode == 1 || !newline[i1%numCenters])));
  float SIZE = linked? 0.1:0.6;
  if(i==1)
    r=rand();

  if(!linked)
    {
      glBindTexture(GL_TEXTURE_2D, texture[0]);   // choose the texture to use.
      glEnable(GL_TEXTURE_2D);
    }
  else
    glDisable(GL_TEXTURE_2D);
  glBegin(GL_QUADS);
  glColor3f(colors[i%3][0] , colors[i%3][1] , colors[i%3][2] );

    //cout<<i<<  " :"<<pts[i].h << " " <<pts[i].v << " " << pts[i].z << endl;
  glTexCoord2f(0,0);
  glVertex3f(pts[i][0]/100, pts[i][1]/100, pts[i][2]/100);
  glTexCoord2f(0,1);
  glVertex3f(pts[i][0]/100+SIZE, pts[i][1]/100, pts[i][2]/100);


  if(linked)
    {
      glVertex3f(pts[i1][0]/100+SIZE, pts[i1][1]/100+SIZE, pts[i1][2]/100);
      glVertex3f(pts[i1][0]/100, pts[i1][1]/100+SIZE, pts[i1][2]/100);
    }
  else
    {
      glTexCoord2f(1,1);
      glVertex3f(pts[i][0]/100+SIZE, pts[i][1]/100+SIZE, pts[i][2]/100);
      glTexCoord2f(1,0);
      glVertex3f(pts[i][0]/100, pts[i][1]/100+SIZE, pts[i][2]/100);

    }
  
  /*    glBegin(GL_POINTS);
	glColor3f(colors[i][0] , colors[i][1] , colors[i][2] );
	glVertex3f(pts[i].h/100, pts[i].v/100, pts[i].z/100);
	glEnd();
  */
    glEnd();

}


void DrawCenter()
{
	
}
void DrawNum(float f, int i)
{
	return;
}
void EraseScreen()
{
  
}

Boolean eTestEnd(void)
{
  return false;
}
int width=600,height=600;
int compar(const void *a, const void *b)
{
  int i = *(int *)a;
  int j = *(int *)b;

  if( pts[i][2] > pts[j][2])
    return 1;
  return -1;
}
extern int gloudness;
void BeginDrawing()
{

  //  qsort (sortedtab, ptsNum,sizeof(int), compar);

  glDisable(GL_TEXTURE_2D);
#if 0  // to debug the beat
  glBegin(GL_QUADS);
  glColor3f(0.3 , 0.3 , 0.5 );
  glVertex3f(-3,-3,0);
  glVertex3f(-3+gloudness/400,-3,0);
  glVertex3f(-3+gloudness/400,-3.1,0);
  glVertex3f(-3,-3.1,0);

  if(beat==1)
    {
      beat--;
      glColor3f(0.3 , 0.3 , 0.5 );
      glVertex3f(-3,3,0);
      glVertex3f(3.5,3,0);
      glVertex3f(3.5,-3.5,0);
      glVertex3f(-3,-3.5,0);
    }
    glEnd();
#endif

  glDisable(GL_DEPTH_TEST);
  glDepthMask(false);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE);

}
void EndDrawing()
{
  usleep(1000);
}

==> dancingparticles0.4.tgz: dancingparticles0.4/gl.cc <==

#include <GL/glut.h>

#include <pthread.h>
#include <sched.h>


#include <math.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include "gl.h"
#include "gl_plugin.h"
#include "etoile.h"



GLfloat heights[64];
extern pthread_mutex_t height_mutex;
extern bool paused;
GLuint texture[2];

void LoadTexture()
{
  static GLubyte imageData[64][64][3];
  for(int i=0;i<64;i++){
    for(int j=0;j<64;j++){
      float x= 255*sqrt((i-32)*(i-32)+(j-32)*(j-32))/32;

      // several formulas have been tested, the better is the "hyperbole"
      //x= 200*(sin(x/56)/(x/56)+0.23);
      //x= 35000*(1/(x+100)-0.0025);
      x=15000*(1/(x+50)-0.003);
      if(x<0)
	x=0;
      imageData[i][j][0]= (GLubyte)x;
      imageData[i][j][1]= (GLubyte)x;
      imageData[i][j][2]= (GLubyte)x;
      
    }
  }
 // Create Texture	
  glGenTextures(1, &texture[0]);
  glBindTexture(GL_TEXTURE_2D, texture[0]);   // 2d texture (x and y size)

  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture
  
  // 2d texture, level of detail 0 (normal), 3 components (red, green, blue), x size from image, y size from image, 
  // border 0 (normal), rgb color data, unsigned byte data, and finally the data itself.
  glTexImage2D(GL_TEXTURE_2D, 0, 3, 64, 64, 0, GL_RGB, GL_UNSIGNED_BYTE,&imageData[0][0][0] );

}
void init_gl(void)
{
  glViewport(0, 0, 640, 480);
  LoadTexture();
  glEnable(GL_TEXTURE_2D);			// Enable Texture Mapping
  glClearColor(0.0f, 0.0f, 1.0f, 0.0f);	// Clear The Background Color To Blue 
  glClearDepth(1.0);				// Enables Clearing Of The Depth Buffer
  glDepthFunc(GL_LESS);			// The Type Of Depth Test To Do
  glEnable(GL_DEPTH_TEST);			// Enables Depth Testing
  glShadeModel(GL_SMOOTH);			// Enables Smooth Color Shading
    
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();				// Reset The Projection Matrix
    
    
  gluPerspective(45.0f,(GLfloat)640/(GLfloat)480,0.1f,100.0f);	// Calculate The Aspect Ratio Of The Window
    
  glMatrixMode(GL_MODELVIEW);
 etoileinit();
    
}
extern int *newline;
extern int numCenters;
extern int gloudness;
extern float spin_x ;
extern float spin_y ;

void draw_gl(void)
{
   static float r=0, SIZE=1;

   float Size =(p.size + p.sizeloudness * gloudness)/2;
   SIZE = (SIZE +Size) /2;
    r++;
  glClearColor(0,0,0,0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(-1, 1, -1, 1, 1.5, 40);
  gluLookAt(0, 0, 6,
	    0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
// BEGIN DRAWING
  glDisable(GL_TEXTURE_2D);

    
#if 0  // to debug the beat which is still bugged..
  glBegin(GL_QUADS);
  glColor3f(0.3 , 0.3 , 0.5 );
  glVertex3f(-3,-3,0);
  glVertex3f(-3+gloudness/400,-3,0);
  glVertex3f(-3+gloudness/400,-3.1,0);
  glVertex3f(-3,-3.1,0);

  if(beat==1)
    {
      beat--;
      glColor3f(0.3 , 0.3 , 0.5 );
      glVertex3f(-3,3,0);
      glVertex3f(-2.5,3,0);
      glVertex3f(-2.5,2.5,0);
      glVertex3f(-3,2.5,0);
    }
    glEnd();
#endif
  glDisable(GL_DEPTH_TEST);
  glDepthMask(false);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE);

  for(int i=0;i<ptsNum;i++)
    {
        GLfloat colors[][3]={ 
        {1.,0.2,0.2   }
        ,{0.2,1.,0.2}
        ,{0.4,0.4,1}};
        
        glBindTexture(GL_TEXTURE_2D, texture[0]);   // choose the texture to use.
        glEnable(GL_TEXTURE_2D);
        glPushMatrix();
        glTranslatef(pts[i][0]/100,pts[i][1]/100,pts[i][2]/100);
        glBegin(GL_QUADS);
        glColor3fv(colors[i%3] );
        
        //cout<<i<<  " :"<<pts[i][0] << " " <<pts[i][1] << " " << pts[i][2] << endl;
        glTexCoord2f(0,0);        glVertex3f(-SIZE, -SIZE, 0);
        glTexCoord2f(1,0);        glVertex3f(SIZE,-SIZE, 0);
        glTexCoord2f(1,1);        glVertex3f(SIZE, SIZE, 0);
        glTexCoord2f(0,1);        glVertex3f(-SIZE, SIZE, 0);
        
        glEnd();
        glPopMatrix();
    }
}
